
package com.aspera.samples.central.job;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.apache.axis2.transport.http.HTTPConstants;

import com.aspera.samples.central.job.jobnet200601.JobNETServiceStub;
import com.aspera.samples.central.job.jobnet200601.JobNETServiceStub.SubmitRequest;
import com.aspera.samples.central.job.jobnet200601.JobNETServiceStub.SubmitResponse;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferFilter;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.FileTransferStatistics;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.GetFileTransferStatisticsRequest;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.GetFileTransferStatisticsResponse;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.GetSessionStatisticsRequest;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.GetSessionStatisticsResponse;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionFilter;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.SessionStatistics;

/**
 * Sample program to submit a transfer job using the JobNET web service and
 * monitoring its progress using the FASPSessionNET web service running on
 * Aspera Central.
 * <p/>
 * 
 * See usage.txt for command line options.
 * 
 * <p/>
 * Copyright 2010 by Aspera, Inc.
 * 
 * @author Aspera
 */
public class TransferJob {

  private String host;
  private int port = 40001;
  private long pollingPeriod = 1000;
  private String remoteHost;
  private String remoteUser;
  private String remotePassword;
  private String operation = "Push";
  private String[] localPaths;
  private String[] remotePaths;

  /**
   * Creates a new instance.
   * 
   * @param args the command-line arguments.
   */
  public TransferJob(String[] args) {
    parseCommandline(args);
    String jobId = submitJob();
    if (jobId != null) {
      monitor(jobId);
    }
  }

  /**
   * Parses the command line and fills in respective member variables.
   * 
   * @param args the command-line arguments.
   */
  private void parseCommandline(String[] args) {
    int i = 0;
    try {
      for (; i < args.length; i++) {
        String opt = args[i].toLowerCase();
        if ("--host".equals(opt)) {
          host = args[++i];
        } else if ("--port".equals(opt)) {
          port = Integer.parseInt(args[++i]);
        } else if ("--period".equals(opt)) {
          pollingPeriod = Long.parseLong(args[++i]);
        } else if ("--remote-host".equals(opt)) {
          remoteHost = args[++i];
        } else if ("--remote-user".equals(opt)) {
          remoteUser = args[++i];
        } else if ("--remote-pass".equals(opt)) {
          remotePassword = args[++i];
        } else if ("--operation".equals(opt)) {
          operation = args[++i];
        } else if ("--local-paths".equals(opt)) {
          localPaths = args[++i].split(",");
        } else if ("--remote-paths".equals(opt)) {
          remotePaths = args[++i].split(",");
        } else {
          throw new Exception();
        }
      }
    } catch (Exception ex) {
      if (i < args.length)
        System.err.println("Invalid option or value [" + args[i]
            + "] at position [" + i + "]");
      else
        System.err.println("Missing value for option [" + args[i - 1] + "]");
      printUsage();
    }

    if (host == null || remoteHost == null || remoteUser == null
        || remotePassword == null || localPaths == null || remotePaths == null) {

      if (host == null)
        System.err.println("Missing --host option.");
      if (remoteHost == null)
        System.err.println("Missing --remote-host option.");
      if (remoteUser == null)
        System.err.println("Missing --remote-user option.");
      if (remotePassword == null)
        System.err.println("Missing --remote-password option.");
      if (localPaths == null)
        System.err.println("Missing --local-paths option.");
      if (remotePaths == null)
        System.err.println("Missing --remote-paths option.");
      System.err.println();
      printUsage();
    }
  }

  /**
   * Starts a transfer and returns its job ID.
   * 
   * @return the ID of the transfer job or null if there was an error.
   */
  private String submitJob() {

    String jobId = null;

    SubmitRequest request = new SubmitRequest();
    request.setType("Aspera.IScpTransfer");
    request.setDefinition(createJobOrder(
        remoteHost,
        remoteUser,
        remotePassword,
        operation,
        localPaths,
        remotePaths));

    try {
      JobNETServiceStub jobService = new JobNETServiceStub("http://" + host
          + ":" + port + "/services/soap/JobNET-200601");

      // disable chunked transfer encoding
      jobService._getServiceClient().getOptions().setProperty(
          HTTPConstants.CHUNKED,
          Boolean.FALSE);

      System.out.print("Submitting transfer job... ");
      SubmitResponse res = jobService.submit(request);
      jobId = res.getJobResult().getID();

      System.out.println("succeeded.");
      System.out.println("Job ID " + jobId);
      System.out.println();
    } catch (Exception e) {
      System.out.println("failed!");
      e.printStackTrace();
    }
    return jobId;
  }

  /**
   * Monitors the given transfer job continously until the job is completed.
   * 
   * @param jobId the job to monitor.
   */
  private void monitor(String jobId) {
    try {
      FASPSessionNETServiceStub service = new FASPSessionNETServiceStub(
          "http://" + host + ":" + port
              + "/services/soap/FASPSessionNET-200911");
      service._getServiceClient().getOptions().setProperty(
          HTTPConstants.CHUNKED,
          Boolean.FALSE);

      System.out.print("Waiting for transfer session to start.");

      boolean sessionCompleted = false;
      SessionStatistics sessionStats = null;
      FileTransferStatistics[] currentTransfers = null;
      boolean first = true;
      String iterationToken = null;
      String sessionUUID = null;

      // create a session filter for our job
      SessionFilter sessionFilter = new SessionFilter();
      sessionFilter.setJobId(new String[] { jobId });

      while (!sessionCompleted) {
        // get the session statistics
        GetSessionStatisticsRequest sreq = new GetSessionStatisticsRequest();
        sreq.setSessionFilter(sessionFilter);
        GetSessionStatisticsResponse sres = service.getSessionStatistics(sreq);

        int resultCount = sres.getSessionStatisticsResult()
            .getResultCount()
            .intValue();

        if (resultCount == 0) {
          // even though the transfer job has been submitted, the corresponding 
          // transfer session has not yet started, so wait a bit and retry
          System.out.print(".");
          Thread.sleep(200);
          continue;
        }

        // get the last session of the set, as there can be multiple sessions if
        // sessions fail and the job is set to retry.
        sessionStats = sres.getSessionStatisticsResult().getSessionStatistics()[resultCount - 1];
        if (!"running".equalsIgnoreCase(sessionStats.getStatus())
            && !"willretry".equalsIgnoreCase(sessionStats.getStatus())) {
          sessionCompleted = true;
        }

        if (!sessionStats.getUUID().equals(sessionUUID)) {
          // this is a new session
          iterationToken = null;
          sessionUUID = sessionStats.getUUID();
        }

        // create a transfer filter with the session UUID and the iteration
        // token. The iteration token allows to retrieve both the ongoing
        // transfers, as well as any newly completed transfers.
        FileTransferFilter transferFilter = new FileTransferFilter();
        transferFilter.setSessionUUID(new String[] { sessionStats.getUUID() });
        transferFilter.setIterationToken(iterationToken);

        // get the transfer statistics
        GetFileTransferStatisticsRequest freq = new GetFileTransferStatisticsRequest();
        freq.setFileTransferFilter(transferFilter);
        GetFileTransferStatisticsResponse fres = service.getFileTransferStatistics(freq);
        currentTransfers = fres.getFileTransferStatisticsResult()
            .getFileTransferStatistics();

        // save the iteration token for the next request
        iterationToken = fres.getFileTransferStatisticsResult()
            .getIterationToken();

        if (first) {
          System.out.println(" started.");
          System.out.println("Session UUID " + sessionStats.getUUID());
          System.out.println();
          System.out.println("S-STATUS     FILES  PROGRESS  F-STATUS    PROGRESS   TRANSFERRED      FILESIZE  PATH");
          System.out.println();
          first = false;
        }

        String sessionInfo = String.format(
            "%-10s %3d/%-3d    %5.1f%% ",
            sessionStats.getStatus(),
            sessionStats.getFileCount().longValue(),
            sessionStats.getExpectedFileCount().longValue(),
            100.0 * sessionStats.getBytesWritten().doubleValue()
                / sessionStats.getExpectedByteCount().doubleValue());
        if ("error".equals(sessionStats.getStatus()))
          sessionInfo += String.format(
              "error %d %s ",
              sessionStats.getErrorCode(),
              sessionStats.getErrorDescription());

        if (currentTransfers != null) {
          for (FileTransferStatistics fti : currentTransfers) {
            System.out.println(String.format(
                "%s %-10s    %5.1f%%  %,12d  %,12d  %s",
                sessionInfo,
                fti.getStatus(),
                fti.getSize().longValue() > 0 ? (double) fti.getBytesWritten()
                    .longValue()
                    / (double) fti.getSize().longValue() * 100.0 : 0.0,
                fti.getBytesWritten().longValue(),
                fti.getSize().longValue(),
                fti.getPath()));
          }
        } else {
          System.out.println(sessionInfo);
        }
        System.out.println();

        // wait for the next polling period
        Thread.sleep(pollingPeriod);
      }
    } catch (Exception ex) {
      System.out.println("Exception while monitoring session!");
      ex.printStackTrace();
    }
  }

  /**
   * Creates the job order XML string.
   * 
   * @param address IP address or host name of the remote host.
   * @param remoteUser the user name on the remote node.
   * @param remotePassword the password of the remote user.
   * @param operation Direction of the transfer [Pull|Push].
   * @param localPaths local file paths.
   * @param remotePaths remote file paths.
   * @return the job order as XML string
   */
  private static String createJobOrder(
      String address,
      String remoteUser,
      String remotePassword,
      String operation,
      String[] localPaths,
      String[] remotePaths) {

    StringBuilder lpaths = new StringBuilder();
    for (String path : localPaths) {
      lpaths.append("<Path>").append(path).append("</Path>");
    }
    StringBuilder rpaths = new StringBuilder();
    for (String path : remotePaths) {
      rpaths.append("<Path>").append(path).append("</Path>");
    }
    
    String jobOrder = 
          "<Order>"  
        +   "<Version>2</Version>"  
        +   "<Agent>aspera.sdk.sample</Agent>" 
        +   "<ApplicationData>" 
        +     "<Cookie>sample cookie</Cookie>"  
        +   "</ApplicationData>"  
        +   "<Operation>" + operation + "</Operation>" 
        +   "<RemoteLocation>"  
        +     "<System>"  
        +       "<Address>" + address + "</Address>" 
        +       "<Port>22</Port>"  
        +       "<UserName>" + remoteUser + "</UserName>" 
        +       "<Password>" + remotePassword + "</Password>"  
        +     "</System>"  
        +     "<Authentication>" 
        +        "<Methods>"  
        +          "<Method>Password</Method>" 
        +        "</Methods>" 
        +     "</Authentication>"  
        +     "<Files>" 
        +        rpaths  
        +     "</Files>"  
        +   "</RemoteLocation>" 
        +   "<LocalLocation>"  
        +     "<Files>" 
        +       lpaths 
        +     "</Files>"  
        +   "</LocalLocation>" 
        +   "<FileParameters>"  
        +     "<PreserveTimestamps>Yes</PreserveTimestamps>" 
        +     "<Overwrite>Diff</Overwrite>"  
        +     "<CreatePath>Yes</CreatePath>" 
        +     "<ResumeCheck>Sparse Checksum</ResumeCheck>" 
        +     "<PreCalculateJobSize>Yes</PreCalculateJobSize>" 
        +     "<RemoveAfterTransfer>No</RemoveAfterTransfer>" 
        +   "</FileParameters>" 
        +   "<LinkParameters>"  
        +     "<RemoteCapacity>2000000</RemoteCapacity>" 
        +     "<LocalCapacity>2000000</LocalCapacity>" 
        +     "<AutoDetectCapacity>No</AutoDetectCapacity>"  
        +   "</LinkParameters>" 
        +   "<RateParameters>" 
        +     "<TargetRate>45000</TargetRate>" 
        +     "<TargetRateAsPercent>No</TargetRateAsPercent>" 
        +     "<MinimumRate>0</MinimumRate>" 
        +     "<MinimumRateAsPercent>No</MinimumRateAsPercent>" 
        +     "<Policy>Fair</Policy>"  
        +     "<Priority>2</Priority>" 
        +     "<BandwidthCap>2000000</BandwidthCap>" 
        +   "</RateParameters>" 
        +   "<SecurityParameters>" 
        +     "<EncryptionCipher>AES-128</EncryptionCipher>" 
        +   "</SecurityParameters>"  
        +   "<RetryParameters>"  
        +     "<Count>3</Count>" 
        +     "<BaseInterval>5</BaseInterval>" 
        +     "<MaximumInterval>10</MaximumInterval>"  
        +   "</RetryParameters>" 
        + "</Order>";

    return jobOrder;
  }

  /**
   * Prints the program usage including valid command line options and exits the
   * program.
   */
  private void printUsage() {
    try {
      BufferedReader reader = new BufferedReader(new InputStreamReader(
          getClass().getResourceAsStream("usage.txt")));
      String line = null;
      while ((line = reader.readLine()) != null)
        System.err.println(line);
    } catch (IOException ex) {
      ex.printStackTrace();
    }
    System.exit(1);
  }

  public static void main(String[] args) throws Exception {
    new TransferJob(args);
  }
}
