
package com.aspera.samples.central.query;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.*;

import org.apache.axis2.databinding.types.PositiveInteger;
import org.apache.axis2.transport.http.HTTPConstants;

import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub;
import com.aspera.samples.central.query.faspsessionnet200911.FASPSessionNETServiceStub.*;

/**
 * Sample program to query the FASPSessionNETService on Aspera Central.
 * <p/>
 * 
 * See usage.txt for command line options.
 * 
 * <p/>
 * Copyright 2010 by Aspera, Inc.
 * 
 * @author Aspera
 */
public class Query {

  private String host;
  private int port = 40001;
  private QueryType queryType = QueryType.SessionStatistics;
  private boolean continuousMode = false;
  private long pollingPeriod = 5000;
  private Date startDate = null;
  private FileTransferFilter filter = new FileTransferFilter();
  private QueryLogger queryLogger = new QueryLogger();

  /**
   * Creates a new instance.
   * 
   * @param args the command-line arguments.
   */
  public Query(String[] args) {
    parseCommandline(args);
    if (continuousMode)
      runContinuous();
    else
      executeQuery();
  }

  /**
   * Parses the command line and fills in respective member variables.
   * 
   * @param args the command-line arguments.
   */
  private void parseCommandline(String[] args) {
    int i = 0;
    try {
      for (; i < args.length; i++) {
        String opt = args[i].toLowerCase();
        if ("--host".equals(opt)) {
          host = args[++i];
        } else if ("--port".equals(opt)) {
          port = Integer.parseInt(args[++i]);
        } else if ("--query".equals(opt)) {
          queryType = QueryType.fromAcronym(args[++i]);
        } else if ("--continuous".equals(opt)) {
          continuousMode = true;
        } else if ("--period".equals(opt)) {
          pollingPeriod = Long.parseLong(args[++i]);
        } else if ("--start".equals(opt)) {
          startDate = new SimpleDateFormat("yyyyMMdd-HHmm").parse(args[++i]);
        } else if ("--maxresults".equals(opt)) {
          filter.setMaxResults(new PositiveInteger(args[++i]));
        } else if ("--sessionstatus".equals(opt)) {
          filter.setSessionStatus(args[++i].split(","));
        } else if ("--sessionuuid".equals(opt)) {
          filter.setSessionUUID(args[++i].split(","));
        } else if ("--cookiefilter".equals(opt)) {
          filter.setCookieFilter(args[++i]);
        } else if ("--direction".equals(opt)) {
          filter.setDirection(args[++i]);
        } else if ("--filenamefilter".equals(opt)) {
          filter.setFilenameFilter(args[++i]);
        } else if ("--transferstatus".equals(opt)) {
          filter.setTransferStatus(args[++i].split(","));
        } else if ("--iterationtoken".equals(opt)) {
          filter.setIterationToken(args[++i]);
        } else {
          throw new Exception();
        }
      }
    } catch (Exception ex) {
      if (i < args.length)
        System.err.println("Invalid option or value [" + args[i]
            + "] at position [" + i + "]");
      else
        System.err.println("Missing value for option [" + args[i - 1] + "]");
      printUsage();
    }

    if (host == null) {
      System.err.println("Missing --host option.\n");
      printUsage();
    }
  }

  /**
   * Continuous query mode.
   */
  private void runContinuous() {
    long lastRun = Long.MIN_VALUE;

    if (startDate != null) {
      // A start date was provided: add it to the query filter.
      Calendar start = Calendar.getInstance();
      start.setTime(startDate);
      filter.setMinSessionStartDate(start);
    }
    if (filter.getIterationToken() == null) {
      // No IterationToken was provided. Default to the special token "MAX" to
      // indicate that we are interested only in sessions/file transfers that
      // change from "now" on. The first query, will therefore only return
      // running sessions/file transfers (if any at all). Subsequent queries
      // will include sessions/file transfers that finish in the future.
      filter.setIterationToken("CURRENT_POSITION");
    }

    while (true) {
      lastRun = System.currentTimeMillis();
      // Execute the query and print out the results. Also assign the returned
      // iteration token to our local variable so it can be reset in the filter
      // in the next loop (line above).
      String iterationToken = executeQuery();
      // Sleep until the next execution.
      sleep(lastRun);
      // set the iterationToken from the previous query for the next query.
      filter.setIterationToken(iterationToken);
    }
  }

  /**
   * Sleeps for {@link #pollingPeriod} milliseconds, adjusting to the execution
   * time of the last query.
   * 
   * @param lastRun the start time of the last query.
   */
  private void sleep(long lastRun) {
    long sleep = pollingPeriod - (System.currentTimeMillis() - lastRun);
    if (sleep > 0) {
      try {
        Thread.sleep(sleep);
      } catch (InterruptedException ex) {
        ex.printStackTrace();
        return;
      }
    }
  }

  /**
   * Executes a query and prints the result to std out.
   * 
   * @return The iteration token returned by the server.
   */
  private String executeQuery() {
    String iterationToken = null;

    try {
      FASPSessionNETServiceStub service = new FASPSessionNETServiceStub(
          "http://" + host + ":" + port
              + "/services/soap/FASPSessionNET-200911");
      service._getServiceClient().getOptions().setProperty(
          HTTPConstants.CHUNKED,
          Boolean.FALSE);

      queryLogger.newQuery(queryType, filter);

      switch (queryType) {
        case SessionStatistics: {
          GetSessionStatisticsRequest req = new GetSessionStatisticsRequest();
          req.setSessionFilter(filter);
          GetSessionStatisticsResponse res = service.getSessionStatistics(req);
          iterationToken = queryLogger.response(res.getSessionStatisticsResult());
          break;
        }
        case SessionInfo: {
          GetSessionInfoRequest req = new GetSessionInfoRequest();
          req.setSessionFilter(filter);
          GetSessionInfoResponse res = service.getSessionInfo(req);
          iterationToken = queryLogger.response(res.getSessionInfoResult());
          break;
        }
        case FileTransferStatistics: {
          GetFileTransferStatisticsRequest req = new GetFileTransferStatisticsRequest();
          req.setFileTransferFilter(filter);
          GetFileTransferStatisticsResponse res = service.getFileTransferStatistics(req);
          iterationToken = queryLogger.response(res.getFileTransferStatisticsResult());
          break;
        }
        case FileTransferInfo: {
          GetFileTransferInfoRequest req = new GetFileTransferInfoRequest();
          req.setFileTransferFilter(filter);
          GetFileTransferInfoResponse res = service.getFileTransferInfo(req);
          iterationToken = queryLogger.response(res.getFileTransferInfoResult());
          break;
        }
        case Info: {
          GetInfoRequest req = new GetInfoRequest();
          req.setFileTransferFilter(filter);
          GetInfoResponse res = service.getInfo(req);
          iterationToken = queryLogger.response(res.getInfoResult());
          break;
        }
        case Statistics: {
          GetStatisticsRequest req = new GetStatisticsRequest();
          req.setFileTransferFilter(filter);
          GetStatisticsResponse res = service.getStatistics(req);
          iterationToken = queryLogger.response(res.getStatisticsResult());
          break;
        }
      }
    } catch (Exception ex) {
      queryLogger.error(ex);
    }

    return iterationToken;
  }

  /**
   * Prints the program usage including valid command line options and exits the
   * program.
   */
  private void printUsage() {
    try {
      BufferedReader reader = new BufferedReader(new InputStreamReader(
          getClass().getResourceAsStream("usage.txt")));
      String line = null;
      while ((line = reader.readLine()) != null)
        System.err.println(line);
    } catch (IOException ex) {
      ex.printStackTrace();
    }
    System.exit(1);
  }

  public static void main(String[] args) {
    new Query(args);
  }
}
